/*
 * prefs.js
 *
 * Copyright 2014, 2015 the Widget Blocker extension copyright holders.
 * See the COPYRIGHT file at the top-level directory of this distribution for a
 * list of copyright holders and how to contact them.
 */

/*
 * This file is part of the Widget Blocker extension for GNOME Shell.
 *
 * Widget Blocker is free software: you are free to use, modify and/or
 * redistribute it for any purpose according to the terms of the LICENSE file
 * found in the top-level directory of this distribution.
 *
 * This program is distributed in the hope that it will be useful, WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the aforementioned LICENSE file for more details.
 */

const Gtk = imports.gi.Gtk;
const Gio = imports.gi.Gio;
const Lang = imports.lang;
const ExtensionUtils = imports.misc.extensionUtils;
const Gettext = imports.gettext.domain('widget_blocker');
const _ = Gettext.gettext;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;
const Lib = Me.imports.lib;

let settings = Convenience.getSettings();

function init() {
    Convenience.initTranslations();
}

const SHORTCUT_COLUMN_KEY = 0;
const SHORTCUT_COLUMN_MODS = 1;

function update_shortcut_row(accel, shortcut_liststore, shortcut_row_iterator) {
    let [key, mods] = (!(typeof accel === 'undefined' || accel === null)) ? Gtk.accelerator_parse(accel) : [0, 0];
    shortcut_liststore.set(
        shortcut_row_iterator,
        [SHORTCUT_COLUMN_KEY, SHORTCUT_COLUMN_MODS],
        [key, mods]
    );
}

function configure_shortcut_treeview(shortcut_treeview, shortcut_liststore, shortcut_row_iterator) {
    let renderer = new Gtk.CellRendererAccel({editable: true});
    renderer.connect('accel-edited', function(renderer, path, key, mods, hw_code) {
        let accel = Gtk.accelerator_name(key, mods);
        update_shortcut_row(accel, shortcut_liststore, shortcut_row_iterator);
        settings.set_strv('actor-picker-shortcut', [accel]);
    });
    renderer.connect('accel-cleared', function() {
        update_shortcut_row(null, shortcut_liststore, shortcut_row_iterator);
        settings.set_strv('actor-picker-shortcut', []);
    });
    let column = new Gtk.TreeViewColumn();
    column.pack_start(renderer, true);
    column.add_attribute(renderer, 'accel-key', SHORTCUT_COLUMN_KEY);
    column.add_attribute(renderer, 'accel-mods', SHORTCUT_COLUMN_MODS);
    return column;
}

function delete_from_list (list, index) {
    let new_list = list.match(/[^;]+|[;]/g).slice(0, 2 * index).toString();
    new_list = new_list.concat(list.match(/[^;]+|[;]/g).slice(2 * (index + 1)).toString());
    if (new_list[new_list.length - 1] === ';') {
        new_list = new_list.match(/[^;]+|[;]/g).slice(0, (2 * index) - 1).toString();
    }
    new_list = new_list.replace(/[,]/g, '');
    return new_list;
}

function new_blacklist_row(name, status, error_status) {
    let status_switch = new Gtk.Switch({active: (status === '1') ? true : false});
    status_switch.connect('notify::active', function(caller) {
        let index = caller.get_parent().get_parent().get_index();
        Lib.set_actor_status(index, caller.active ? '1' : '0');
        settings.set_boolean('prefs-modification', !(settings.get_boolean('prefs-modification')));
    });
    let label = new Gtk.Label({label: name});
    if (error_status === '1') {
        label.label = _("error").concat(': ', label.label);
    }
    let forget_button = new Gtk.Button({label: _("Forget")});
    forget_button.connect('clicked', function(caller) {
        let index = caller.get_parent().get_parent().get_index();
        Lib.set_actor_status(index, '0');
        settings.set_boolean('prefs-modification',!(settings.get_boolean('prefs-modification')));
        settings.set_string('blacklisted-actors-statuses',
            delete_from_list(
                settings.get_string('blacklisted-actors-statuses'),
                index
            )
        );
        settings.set_string('blacklisted-actors-error-statuses',
            delete_from_list(
                settings.get_string('blacklisted-actors-error-statuses'),
                index
            )
        );
        settings.set_string('blacklisted-actors',
            delete_from_list(
                settings.get_string('blacklisted-actors'),
                index
            )
        );
        caller.get_parent().get_parent().destroy();
    });
    let horizontal_box = new Gtk.Box({
        orientation: Gtk.Orientation.HORIZONTAL,
        spacing: 20,
        margin_left: 12,
        margin_right: 12
    });
    horizontal_box.pack_start(status_switch, false, false, 0);
    horizontal_box.add(label);
    horizontal_box.pack_end(forget_button, false, false, 0);
    let row = new Gtk.ListBoxRow();
    row.add(horizontal_box);
    return row;
}

let listbox;

function reload_blacklist_ui(listbox) {
    let actors = [
        settings.get_string('blacklisted-actors'),
        settings.get_string('blacklisted-actors-statuses'),
        settings.get_string('blacklisted-actors-error-statuses')
    ];
    for (let i = 0; i < 3; ++i) {
        if (actors[i] === '' || actors[i] === '\'\'') {
            return;
        }
    }
    actors[0] = actors[0].split(';');
    actors[1] = actors[1].split(';');
    actors[2] = actors[2].split(';');
    for (let i = 0; i < actors[0].length; ++i) {
        listbox.add(new_blacklist_row(actors[0][i], actors[1][i], actors[2][i]));
    }
    listbox.show_all();
}

/*
 * a weird bug was making the callback on the 'changed::blacklist-addition' signal run
 * twice per signal firing, so we introduce this global flag to prevent duplication.
 */
let last_actor = settings.get_string('blacklisted-actors').split(';');
if (last_actor.length > 1) {
    last_actor = last_actor[last_actor.length - 1];
}

function buildPrefsWidget() {
    let builder = new Gtk.Builder();
    builder.add_from_file(Me.path + '/prefs.ui');
    let shortcut_treeview = builder.get_object('shortcut_treeview');
    let shortcut_liststore = builder.get_object('shortcut_liststore');
    let shortcut_switch = builder.get_object('switch');
    shortcut_switch.active = settings.get_boolean('actor-picker-shortcut-status');
    let shortcut_row_iterator = shortcut_liststore.append();
    shortcut_treeview.append_column(
        configure_shortcut_treeview(shortcut_treeview, shortcut_liststore, shortcut_row_iterator)
    );
    update_shortcut_row(
        settings.get_strv('actor-picker-shortcut')[0],
        shortcut_liststore,
        shortcut_row_iterator
    );
    settings.bind('actor-picker-shortcut-status', shortcut_switch, 'active', Gio.SettingsBindFlags.DEFAULT);
    settings.connect('changed::actor-picker-shortcut', function() {
        update_shortcut_row(
            settings.get_strv('actor-picker-shortcut')[0],
            shortcut_liststore,
            shortcut_row_iterator
        );
    });
    let scrollbar_adjustment = builder.get_object('scrolledwindow').get_vadjustment();
    listbox = builder.get_object('blacklist');
    reload_blacklist_ui(listbox);
    listbox.set_adjustment(scrollbar_adjustment);
    settings.connect('changed::blacklist-addition', Lang.bind(listbox, function() {
        let actor = settings.get_string('blacklisted-actors').split(';');
        actor = actor[actor.length - 1];
        if (actor != last_actor) {
            listbox.add(new_blacklist_row(actor, '1', '0'));
            listbox.show_all();
            last_actor = actor;
        }
    }));
    return builder.get_object('main_container');
}
